package com.jz.sms.channel.chuanglan.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.jz.sms.channel.chuanglan.config.ContentEnum;
import com.jz.util.HttpUtil;
import com.jz.sms.channel.chuanglan.dto.*;
import com.jz.sms.channel.chuanglan.service.ChuangLanSmsService;
import com.jz.sms.channel.chuanglan.utils.ChuangLanSmsUtil;
import com.jz.sms.channel.chuanglan.config.ChuangLanSMSConfig;
import com.jz.sms.config.ResponseCode;
import com.jz.sms.channel.dmHub.service.DmHubService;
import com.jz.sms.repository.domain.SmsTemplateInfo;
import com.jz.sms.service.SmsTemplateService;
import com.jz.util.Md5Util;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;


@SuppressWarnings("DuplicatedCode")
@Service
public class ChuangLanSmsServiceImpl implements ChuangLanSmsService {

    private static final Logger log = LoggerFactory.getLogger(ChuangLanSmsServiceImpl.class);
    private static final String SUCCESS = "success";
    private static final String sendSuccessCode = "0";  // 发送响应成功字段
    private static final String createTemplateStatus = "status"; // 响应状态字段
    private static final String retMsg = "msg";                 //  响应描述
    private static final String createTemplateData = "data";    //  创建模板响应数据

    private static final String notificationId = "49"; // 创蓝通知类短信ID
    private static final String marketingId = "52";    // 创蓝营销类短信ID

    @Autowired
    private SmsTemplateService templateService;
    @Autowired
    private DmHubService dmHubService;

    @Override
    public CLBizResponse createTemplate(String content, String type) throws Exception {
        log.info("进入创蓝创建短信接口模板的逻辑：{}", content);
        Map<String, Object> requestMap = new HashMap<>();
        String appId;
        String username = ChuangLanSMSConfig.interfaceUser;
        String password = ChuangLanSMSConfig.interfacePassword;
        if (ContentEnum.SMS_TYPE_NOTIFICATION.getValue().equals(type)) {
            appId = notificationId;
        } else if (ContentEnum.SMS_TYPE_MARKETING.getValue().equals(type)) {
            appId = marketingId;
            // 两个系统模式不同，DMHUB会带上回TD退订，但是创蓝回自动追加回TD退订。只是营销类
            content = content.replaceAll("回TD退订", "");
        } else {
            // TODO 发送警告
            log.error("错误，传入的模板类型未知，类型为：{}", type);
            return new CLBizResponse(ResponseCode.SYSTEM_ERROR, "未知的模板类型:" + type, 0);
        }

        String timestamp = String.valueOf(System.currentTimeMillis() / 1000);
        String encrypt = Md5Util.encrypt(new StringBuffer(username).append(password).append(timestamp).toString());
        requestMap.put("username", username);
        requestMap.put("timestamp", timestamp);
        requestMap.put("signature", encrypt);
        requestMap.put("appid", appId);
        requestMap.put("unsubscribe", "回TD退订");
        requestMap.put("content", content);
        log.info("请求创蓝创建短信模板的请求参数为：{}", requestMap);
        String retStr = HttpUtil.doPost(ChuangLanSMSConfig.addTemplate, requestMap);
        log.info("请求创蓝创建短信模板的返回数据为：{}", retStr);
        JSONObject retJson = JSONObject.parseObject(retStr);
        String status = retJson.getString(createTemplateStatus);
        if (SUCCESS.equals(status)) {
            JSONObject data = retJson.getJSONObject(createTemplateData);
            return new CLBizResponse(ResponseCode.SUCCESS, data);
        } else {
            String msg = retJson.getString(retMsg);
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), msg);
        }
    }

    @Override
    public CLBizResponse sendByFixed(JSONObject uidJson, String content, String sendParams, String type) throws Exception {
        String account;
        String password;
        if (ContentEnum.SMS_TYPE_NOTIFICATION.getValue().equals(type)) {
            account = ChuangLanSMSConfig.notificationAccount;
            password = ChuangLanSMSConfig.notificationPassword;
            log.info("发送短信的模式为：{通知类}");
        } else if (ContentEnum.SMS_TYPE_MARKETING.getValue().equals(type)) {
            account = ChuangLanSMSConfig.marketingAccount;
            password = ChuangLanSMSConfig.marketingPassword;
            log.info("发送短信的模式为：{营销类}");
        } else {
            log.error("发送短信的模式为：{未知类型}");
            return new CLBizResponse(ResponseCode.UNKNOWN_TEMPLATE_TYPE);
        }

        CLSendFixedRequest request = new CLSendFixedRequest(account, password, content, sendParams, "true", uidJson.toJSONString());

        String requestJson = JSONObject.toJSONString(request);
        String batchId = uidJson.getString("dmHubId");
        log.info("【{}】发送固定内容短信,请求上游数据为：{}", batchId, requestJson);
        String responseStr = ChuangLanSmsUtil.sendSmsByPost(ChuangLanSMSConfig.sendFixed, requestJson);
        log.info("【{}】发送固定内容短信,上游返回数据为：{}", batchId, requestJson);

        if (StringUtils.isBlank(responseStr)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        CLSendSMSResponse response = JSON.parseObject(responseStr, CLSendSMSResponse.class);

        String code = response.getCode();
        String errorMsg = response.getErrorMsg();
        String msgId = response.getMsgId();
        if (!sendSuccessCode.equals(code)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), errorMsg);
        } else {
            JSONObject data = new JSONObject();
            data.put("msgId", msgId);
            return new CLBizResponse(ResponseCode.SUCCESS, data);
        }
    }

    @Override
    public CLBizResponse sendByVariable(JSONObject uidJson, String content, String params, String type) throws Exception {
        String account;
        String password;
        if (ContentEnum.SMS_TYPE_NOTIFICATION.getValue().equals(type)) {
            account = ChuangLanSMSConfig.notificationAccount;
            password = ChuangLanSMSConfig.notificationPassword;
            log.info("发送短信的模式为：{通知类}");
        } else if (ContentEnum.SMS_TYPE_MARKETING.getValue().equals(type)) {
            account = ChuangLanSMSConfig.marketingAccount;
            password = ChuangLanSMSConfig.marketingPassword;
            log.info("发送短信的模式为：{营销类}");
        } else {
            log.info("发送短信的模式为：{未知类型},type:{}", type);
            return new CLBizResponse(ResponseCode.UNKNOWN_TEMPLATE_TYPE);
        }
        CLSendVariableRequest request = new CLSendVariableRequest(account, password, content, params, "true", uidJson.toJSONString());
        String requestJson = JSONObject.toJSONString(request);
        String batchId = uidJson.getString("dmHubId");
        log.info("【{}】发送变量内容短信,请求上游数据为：{}，\r\n请求地址为：{}", batchId, requestJson, ChuangLanSMSConfig.sendVariable);
        String responseStr = ChuangLanSmsUtil.sendSmsByPost(ChuangLanSMSConfig.sendVariable, requestJson);
        log.info("【{}】发送变量内容短信,上游返回数据为：{}", batchId, responseStr);

        if (StringUtils.isBlank(responseStr)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_BLANK);
        }
        CLSendSMSResponse response = JSON.parseObject(responseStr, CLSendSMSResponse.class);
        String code = response.getCode();
        String errorMsg = response.getErrorMsg();
        String successNum = response.getSuccessNum();
        String failNum = response.getFailNum();
        String msgId = response.getMsgId();
        if (!sendSuccessCode.equals(code)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_FAIL.getCode(), errorMsg);
        } else {
            JSONObject data = new JSONObject();
            data.put("successNum", successNum);
            data.put("failNum", failNum);
            data.put("msgId", msgId);
            return new CLBizResponse(ResponseCode.SUCCESS, data);
        }
    }


    @Override
    public CLBizResponse asyncNotify(CLNotifyRequest notifyInfo) throws Exception {
        String action = notifyInfo.getAction();
        if (!"messageModel".equals(action)) {
            return new CLBizResponse(ResponseCode.UPSTREAM_UNKNOWN_MODEL);
        }
        String id = notifyInfo.getId();
        String status = notifyInfo.getAuditStatus();
        if (!"1".equals(status)) {
            String msg = notifyInfo.getAuditReason();
            boolean update = templateService.updateForSet("status = abnormal,up_reject_msg = " + msg, new EntityWrapper<SmsTemplateInfo>().eq("up_template_id", id));
            // TODO 发送告警,模板审核未通过
            log.error("短信模板审核不通过，原因：{}", msg);
        }
        return new CLBizResponse(ResponseCode.SUCCESS);
    }


    @Override
    public CLBizResponse report(ReportDto report) throws Exception {
        // TODO 状态报告要特殊处理，一个个接受，批量发送（处理方式待定）
        dmHubService.smsReport(report);
        return new CLBizResponse(ResponseCode.SUCCESS);
    }


}
